/* libc-internal interface for mutex locks.  Stub version.
   Copyright (C) 1996,97,99,2000,01 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307 USA.  */

#ifndef _BITS_LIBC_LOCK_H
#define _BITS_LIBC_LOCK_H 1

typedef struct rts_mask_t
{
  volatile int mask;            /*!< Atomic indication that we are in a critical section */
  volatile int pending;         /*!< Specifies if interrupt occurred during a user-mode critical section */
           int refcount;        /*!< Number of nested user-mode critical sections */
} rts_mask_t;                   /*!< Struct for tracking the state of the rts_locks */
typedef int rts_lock_t;

/* defined in sysdeps/blrts/start.c */
extern void rts_lock_init(rts_lock_t *lock);
extern void rts_lock_fini(rts_lock_t *lock);
extern void rts_lock_lock(rts_lock_t *lock);
extern int  rts_lock_try(rts_lock_t *lock);
extern void rts_lock_unlock(rts_lock_t *lock);
extern rts_mask_t * rts_lock_register_handler (void (*f) (void *), void *fArg);

/* Define a lock variable NAME with storage class CLASS.  The lock must be
   initialized with __libc_lock_init before it can be used (or define it
   with __libc_lock_define_initialized, below).  Use `extern' for CLASS to
   declare a lock defined in another module.  In public structure
   definitions you must use a pointer to the lock structure (i.e., NAME
   begins with a `*'), because its storage size will not be known outside
   of libc.  */
#define __libc_lock_define(CLASS,NAME)                           CLASS rts_lock_t NAME;
#define __libc_lock_define_recursive(CLASS,NAME)                 CLASS rts_lock_t NAME;
#define __libc_rwlock_define(CLASS,NAME)                         CLASS rts_lock_t NAME;

/* Define an initialized lock variable NAME with storage class CLASS.  */
#define __libc_lock_define_initialized(CLASS,NAME)               CLASS rts_lock_t NAME;
#define __libc_rwlock_define_initialized(CLASS,NAME)             CLASS rts_lock_t NAME;
#define __libc_lock_define_initialized_recursive(CLASS,NAME)     CLASS rts_lock_t NAME;
#define  _LIBC_LOCK_RECURSIVE_INITIALIZER 0

/* Initialize the named lock variable, leaving it in a consistent, unlocked state.  */
#define __libc_lock_init(NAME)                                   rts_lock_init(&(NAME))
#define __libc_rwlock_init(NAME)                                 rts_lock_init(&(NAME))
#define __libc_lock_init_recursive(NAME)                         rts_lock_init(&(NAME))

/* Finalize the named lock variable, which must be locked.  It cannot be
   used again until __libc_lock_init is called again on it.  This must be
   called on a lock variable before the containing storage is reused.  */
#define __libc_lock_fini(NAME)                                   rts_lock_fini(&(NAME))
#define __libc_rwlock_fini(NAME)                                 rts_lock_fini(&(NAME))
#define __libc_lock_fini_recursive(NAME)                         rts_lock_fini(&(NAME))

/* Lock the named lock variable.  */
#define __libc_lock_lock(NAME)                                   rts_lock_lock(&(NAME))
#define __libc_rwlock_rdlock(NAME)                               rts_lock_lock(&(NAME))
#define __libc_rwlock_wrlock(NAME)                               rts_lock_lock(&(NAME))
#define __libc_lock_lock_recursive(NAME)                         rts_lock_lock(&(NAME))

/* Try to lock the named lock variable.  */
#define __libc_lock_trylock(NAME)                                rts_lock_try(&(NAME))
#define __libc_rwlock_tryrdlock(NAME)                            rts_lock_try(&(NAME))
#define __libc_rwlock_trywrlock(NAME)                            rts_lock_try(&(NAME))
#define __libc_lock_trylock_recursive(NAME)                      rts_lock_try(&(NAME))

/* Unlock the named lock variable.  */
#define __libc_lock_unlock(NAME)                                 rts_lock_unlock(&(NAME))
#define __libc_rwlock_unlock(NAME)                               rts_lock_unlock(&(NAME))
#define __libc_lock_unlock_recursive(NAME)                       rts_lock_unlock(&(NAME))


/* rtld lock */
#define __rtld_lock_define_initialized_recursive __libc_lock_define_initialized
#define __rtld_lock_init_recursive __libc_lock_init
#define __rtld_lock_fini_recursive __libc_lock_fini
#define __rtld_lock_trylock_recursive __libc_lock_trylock
#define __rtld_lock_unlock_recursive __libc_lock_unlock
#define __rtld_lock_lock_recursive __libc_lock_lock
#define __rtld_lock_define_recursive __libc_lock_define_recursive

/* Define once control variable.  */
#define __libc_once_define(CLASS, NAME) CLASS int NAME = 0

/* Call handler iff the first call.  */
#define __libc_once(ONCE_CONTROL, INIT_FUNCTION) \
  do {									      \
    if ((ONCE_CONTROL) == 0) {						      \
      INIT_FUNCTION ();							      \
      (ONCE_CONTROL) = 1;						      \
    }									      \
  } while (0)

/* Start a critical region with a cleanup function */
#define __libc_cleanup_region_start(DOIT, FCT, ARG)			    \
{									    \
  typeof (***(FCT)) *__save_FCT = (DOIT) ? (FCT) : 0;			    \
  typeof (ARG) __save_ARG = ARG;					    \
  /* close brace is in __libc_cleanup_region_end below. */

/* End a critical region started with __libc_cleanup_region_start. */
#define __libc_cleanup_region_end(DOIT)					    \
  if ((DOIT) && __save_FCT != 0)					    \
    (*__save_FCT)(__save_ARG);						    \
}

/* Sometimes we have to exit the block in the middle.  */
#define __libc_cleanup_end(DOIT)					    \
  if ((DOIT) && __save_FCT != 0)					    \
    (*__save_FCT)(__save_ARG);						    \

#define __libc_cleanup_push(fct, arg) __libc_cleanup_region_start (1, fct, arg)
#define __libc_cleanup_pop(execute) __libc_cleanup_region_end (execute)

/* We need portable names for some of the functions.  */
#define  __libc_mutex_unlock rts_lock_unlock

/* Type for key of thread specific data.  */
typedef int __libc_key_t;

/* Create key for thread specific data.  */
#define __libc_key_create(KEY,DEST) -1

/* Set thread-specific data associated with KEY to VAL.  */
#define __libc_setspecific(KEY,VAL) ((void)0)

/* Get thread-specific data associated with KEY.  */
#define __libc_getspecific(KEY) 0

#ifndef _LIBC_REENTRANT
static void cancel_handler(void * x) { }
#endif

#endif	/* bits/libc-lock.h */
