/* Determine current working directory.  Linux version.
   Copyright (C) 1997, 1998, 1999, 2000 Free Software Foundation, Inc.
   This file is part of the GNU C Library.
   Contributed by Ulrich Drepper <drepper@cygnus.com>, 1997.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307 USA.  */

/* Copied from gnu/glibc-2.2.5/sysdeps/unix/sysv/linux/getcwd.c and modified for blrts. */

#include <assert.h>
#include <errno.h>
#include <limits.h>
#include <stdlib.h>
#include <unistd.h>

#include <sysdep.h>
#include <sys/syscall.h>
#include <bp-checks.h>

extern int __syscall_getcwd(char *buf, size_t size);

char *
__getcwd (char *buf, size_t size)
{
  char *path;
  size_t alloc_size = size;
  int retval;

  if (size == 0)  {
     if (buf != NULL) {
        __set_errno (EINVAL);
        return NULL;
     }

     alloc_size = PATH_MAX;
  }

  if (buf != NULL) {
     path = buf;
  }
  else {
      path = malloc (alloc_size);
      if (path == NULL) {
         return NULL;
      }
  }

  retval = INLINE_SYSCALL (getcwd, 2, CHECK_STRING (path), alloc_size);
  if (retval >= 0) {
  	if (buf == NULL && size == 0) {
        	/* Ensure that the buffer is only as large as necessary.  */
        	buf = realloc (path, (size_t) retval);
	}

     if (buf == NULL) {
        /* Either buf was NULL all along, or `realloc' failed but
           we still have the original string.  */
        buf = path;
     }

	  return buf;
  }

  /* It should never happen that the `getcwd' syscall failed because
     the buffer is too small if we allocated the buffer ourselves
	  large enough.  */
  assert (errno != ERANGE || buf != NULL || size != 0);

  if (buf == NULL) {
     free (path);
  }

  return NULL;
}

weak_alias (__getcwd, getcwd)
