/* Copyright (C) 1991, 1992, 1993, 1996, 1997 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307 USA.  */

/* Copied from gnu/glibc-2.3.6/sysdeps/posix/sleep.c and modified for blrts */
         
#include <signal.h>
#include <time.h>
#include <sys/time.h>
#include <unistd.h>
#include <errno.h>

/* Make the process sleep for SECONDS seconds.  Note that blrts does not
   support 'sigsuspend' so 'sleep' does not return if a signal arrives
   and it is handled.  This behavior is different than other systems but
   a blrts application really shouldn't be using 'sleep' in the first place.
   There is no return value to indicate error, but if `sleep' returns 
   SECONDS, it probably didn't work.  */
unsigned int
__sleep (unsigned int seconds)
{
  struct timeval now;
  time_t endtime;

  if (seconds == 0)
    return 0;

  /* Get the curent time and calculate the time when 'sleep' needs to return.
     Note that since 'time' also uses 'gettimeofday' and does NOT use the
     microseconds field to round up the time to the nearest second, 'sleep'
     does not round either. */
  if (gettimeofday(&now, NULL) != 0) {
    return seconds;
  }
  endtime = now.tv_sec + seconds;

  /* Spin until the current time is equal to or greater than the calculated 
     end time. */
  while (now.tv_sec < endtime) {
    gettimeofday(&now, NULL);
  }

  return 0;
}
weak_alias (__sleep, sleep)

