/* Entrypoint for programs running under blrts. */

#if 1

/* During the build of glibc, we get "undefined reference to _init, _fini" when
 * linking the "iconvconfig" program, because for some reason this program
 * is built using "-nostdlib -nostartfiles".
 * The following nonsense provides a workaround until I can figure
 * out why that program is needed and why they're linking it that way. --DL
 */

void
_dummyinit (void)
{
}

void
_dummyfini (void)
{
}

weak_alias (_dummyinit, _init);
weak_alias (_dummyfini, _fini);
#endif

#include <fpu_control.h>
#include <libc-internal.h>
#include <stdlib.h>
#include <set-hooks.h>

DEFINE_HOOK_RUNNER (__libc_subinit, __libc_init,
                    (int argc, char **argv, char **envp), (argc, argv, envp));

extern char **__environ;
extern int __libc_multiple_libcs;

/* Remember the command line argument and enviroment contents */
int __libc_argc;
char **__libc_argv;


extern void __libc_init (int argc, char **argv, char **env);
extern void _init (void);
extern void _fini (void);
extern int main (int argc, char **argv, char **env);
extern void exit (int rc);
extern int atexit (void (*function) (void));
extern int linux_execve (const char *file, char **argv, char **envp);
extern char *getenv (const char *name);

asm ("\n\
	.section .text					\n\
	.align 2					\n\
	.globl _start					\n\
	.type _start,@function				\n\
_start:							\n\
	cmpwi 0,3,0					\n\
	bne 0,_start_blrts				\n\
	lwz 3,0(1)					\n\
	addi 4,1,4					\n\
	stwu 1,-16(1)					\n\
	li 0,0						\n\
	stw 0,0(1)					\n\
	bl _start_linux					\n\
._start:						\n\
	.size	_start,._start-_start			\n\
	.align 2					\n\
	.globl linux_execve				\n\
	.type	linux_execve,@function			\n\
linux_execve:						\n\
	li 0,11						\n\
	sc						\n\
	bnslr						\n\
	li 3,-1						\n\
	li 0,1						\n\
	sc						\n\
.linux_execve:						\n\
	.size	linux_execve,.linux_execve-linux_execve	\n\
	.previous					\n");

void
_start_blrts (int argc, char **argv)
{
  /* do this to prevent sbrk.c from reinitializing the break multiple times */
  __libc_multiple_libcs = 0;

  /* Set the FPU control word to the proper default value if the
     kernel would use a different value.  (In a static program we
     don't have this information.)  */
  __setfpucw (__fpu_control);

  /* Save the command-line arguments.  */
  __libc_argc = argc;
  __libc_argv = argv;
  __environ = &argv[argc + 1];

  /* initialize c library */
  /* __libc_init(argc, argv, __environ); */
  __init_misc (argc, argv, __environ);

  atexit (_fini);

  /* initialize c++ static objects */
  _init ();

  /* run the program */
  exit (main (argc, argv, __environ));
}

void
_start_linux (int argc, char **argv)
{
  /* If we're being executed on a Linux machine and the USER
   * has defined a CIORUN env var, then redirect this app to
   * our pseudo BLRTS loader which just exec's ciorun for us.
   * Otherwise, just execute the app as we normally would.
   */
  __environ = &argv[argc + 1];
  if (getenv ("CIORUN") != 0)
    linux_execve ("/lib/ld-blrts.so", argv, __environ);
  _start_blrts (argc, argv);
}

/* The following is an experiment to allow user level code to be interrupted
 * within "safe" regions, in order to run a function with higher priority.
 *
 * This work is motivated by George and Xavi's need to run an MPI "advance" function
 * whenever packets materialize in tree and torus fifos.
 *
 * Rather than implementing a full blown pthreads package,
 * we're trying something simpler:
 *
 *  - compile glibc using -D_LIBC_REENTRANT so that the existing __libc_lock()
 *    __libc_unlock() functions will protect non-reentrant system resources
 *    from interrupts
 *
 *  - allow a user-level signal handler to trigger an "advance" function to
 *    be executed as soon as the system enters a "safe" (reentrant) region of
 *    mainline code
 *
 * 2004 Jun 27 Derek Lieber
 */

#include <bits/libc-lock.h>

static rts_mask_t GlobalMask = { 0, 0, 0 };     /*!< ??? */
static void (*DeferredFunction) (void *) = NULL;        /*!< function to run as soon as mainline enters a "safe" region */
static void *DeferredFunctionArg = NULL;        /*!< ??? */

/* Rts locking interface. */
void
rts_lock_init (rts_lock_t * lock)
{
}

void
rts_lock_fini (rts_lock_t * lock)
{
}

/* Enter an unsafe region. */
void
rts_lock_lock (rts_lock_t * lock)
{
  GlobalMask.mask = 1;
  GlobalMask.refcount += 1;
}

/* Enter an unsafe region. */
int
rts_lock_try (rts_lock_t * lock)
{
  GlobalMask.mask = 1;
  GlobalMask.refcount += 1;
  return 0;
}

/* Leave an unsafe region. */
void
rts_lock_unlock (rts_lock_t * lock)
{
  GlobalMask.refcount -= 1;

  if (GlobalMask.refcount < 0)
    {
#define MASK_ERROR "--> GlobalMask.refcount < 0\n"
      write (2, MASK_ERROR, sizeof (MASK_ERROR) - 1);
      exit (1);
    }

  if (0 == GlobalMask.refcount)
    {
      GlobalMask.mask = 0;
      if (GlobalMask.pending)
        {
          GlobalMask.pending = 0;
          if (DeferredFunction)
            DeferredFunction(DeferredFunctionArg);
        }
    }
}


rts_mask_t *
rts_lock_register_handler (void (*f) (void *), void *fArg)
{
  DeferredFunction = f;
  DeferredFunctionArg = fArg;
  return &GlobalMask;
}
