/****************************************************************************/
/* ZEPTOOS:zepto-info */
/*     This file is part of ZeptoOS: The Small Linux for Big Computers.
 *     See www.mcs.anl.gov/zeptoos for more information.
 */
/* ZEPTOOS:zepto-info */
/* */
/* ZEPTOOS:zepto-fillin */
/*     $Id: fcntl.c,v 1.4 2007/06/27 15:20:38 iskra Exp $
 *     ZeptoOS_Version: 1.2
 *     ZeptoOS_Heredity: FOSS_MOD
 *     ZeptoOS_License: GPL
 */
/* ZEPTOOS:zepto-fillin */
/* */
/* ZEPTOOS:zepto-gpl */
/*      Copyright: Argonne National Laboratory, Department of Energy,
 *                 and UChicago Argonne, LLC.  2004, 2005, 2006, 2007
 *      ZeptoOS License: GPL
 * 
 *      This software is free.  See the file ZeptoOS/misc/license.GPL
 *      for complete details on your rights to copy, modify, and use this
 *      software.
 */
/* ZEPTOOS:zepto-gpl */
/****************************************************************************/

#include <fcntl.h>
#include <stdarg.h>

#include "unix.h"

int
__libc_fcntl (int fd, int cmd, ...)
{
  va_list ap;
  void *arg;
  void *arg_ptr;
  int arg_len;

  va_start (ap, cmd);
  arg = va_arg (ap, void *);
  va_end (ap);

  switch (cmd)
  {
      case F_GETFL:
	  /* Does not take any argument.  */
	  arg_ptr = NULL;
	  arg_len = 0;
	  break;

      case F_SETFL:
      case F_DUPFD:
	  /* "arg" is actually a long int, not a pointer.  */
	  arg_ptr = &arg;
	  arg_len = sizeof(long);
	  break;

      case F_GETLK:
      case F_SETLK:
      case F_SETLKW:
	  arg_ptr = arg;
	  arg_len = sizeof(struct flock);
	  break;

      case F_GETLK64:
      case F_SETLK64:
      case F_SETLKW64:
	  arg_ptr = arg;
	  arg_len = sizeof(struct flock64);
	  break;

      default:
	  __set_errno(EINVAL);
	  return -1;
  }

  return unix_decode_result(unix_fcntl(fd, cmd, arg_ptr, &arg_len));
}

weak_alias (__libc_fcntl, __fcntl)
weak_alias (__libc_fcntl, fcntl)
