/****************************************************************************/
/* ZEPTOOS:zepto-info */
/*     This file is part of ZeptoOS: The Small Linux for Big Computers.
 *     See www.mcs.anl.gov/zeptoos for more information.
 */
/* ZEPTOOS:zepto-info */
/* */
/* ZEPTOOS:zepto-fillin */
/*     $Id: getcwd.c,v 1.3 2007/06/27 15:20:38 iskra Exp $
 *     ZeptoOS_Version: 1.2
 *     ZeptoOS_Heredity: FOSS_MOD
 *     ZeptoOS_License: GPL
 */
/* ZEPTOOS:zepto-fillin */
/* */
/* ZEPTOOS:zepto-gpl */
/*      Copyright: Argonne National Laboratory, Department of Energy,
 *                 and UChicago Argonne, LLC.  2004, 2005, 2006, 2007
 *      ZeptoOS License: GPL
 * 
 *      This software is free.  See the file ZeptoOS/misc/license.GPL
 *      for complete details on your rights to copy, modify, and use this
 *      software.
 */
/* ZEPTOOS:zepto-gpl */
/****************************************************************************/

/* Copied from sysdeps/blrts/getcwd.c and modified for zoid. */

#include <assert.h>
#include <errno.h>
#include <limits.h>
#include <stdlib.h>
#include <unistd.h>

#include "unix.h"

char *
__getcwd (char *buf, size_t size)
{
  char *path;
  size_t alloc_size = size;
  int retval;

  if (size == 0)  {
     if (buf != NULL) {
        __set_errno (EINVAL);
        return NULL;
     }

     alloc_size = PATH_MAX;
  }

  if (buf != NULL) {
     path = buf;
  }
  else {
      path = malloc (alloc_size);
      if (path == NULL) {
         return NULL;
      }
  }

  retval = unix_decode_result(unix_getcwd(path, alloc_size));
  if (retval >= 0) {
	  if (buf == NULL && size == 0) {
        /* Ensure that the buffer is only as large as necessary.  */
        buf = realloc (path, (size_t) retval);
     }

     if (buf == NULL) {
        /* Either buf was NULL all along, or `realloc' failed but
           we still have the original string.  */
        buf = path;
     }

	  return buf;
  }

  /* It should never happen that the `getcwd' syscall failed because
     the buffer is too small if we allocated the buffer ourselves
	  large enough.  */
  assert (errno != ERANGE || buf != NULL || size != 0);

  if (buf == NULL) {
     free (path);
  }

  return NULL;
}

weak_alias (__getcwd, getcwd)
