/****************************************************************************/
/* ZEPTOOS:zepto-info */
/*     This file is part of ZeptoOS: The Small Linux for Big Computers.
 *     See www.mcs.anl.gov/zeptoos for more information.
 */
/* ZEPTOOS:zepto-info */
/* */
/* ZEPTOOS:zepto-fillin */
/*     $Id: pread64.c,v 1.6 2007/06/27 15:20:38 iskra Exp $
 *     ZeptoOS_Version: 1.2
 *     ZeptoOS_Heredity: FOSS_MOD
 *     ZeptoOS_License: GPL
 */
/* ZEPTOOS:zepto-fillin */
/* */
/* ZEPTOOS:zepto-gpl */
/*      Copyright: Argonne National Laboratory, Department of Energy,
 *                 and UChicago Argonne, LLC.  2004, 2005, 2006, 2007
 *      ZeptoOS License: GPL
 * 
 *      This software is free.  See the file ZeptoOS/misc/license.GPL
 *      for complete details on your rights to copy, modify, and use this
 *      software.
 */
/* ZEPTOOS:zepto-gpl */
/****************************************************************************/

#include <errno.h>
#include <stddef.h>
#include <unistd.h>

#include "unix.h"

/* Note: most of the code below is duplicated in pwrite64.c, and plain/p
   variants of write and read.  If anything needs to be changed in one of
   them, it probably also should be changed in all the others!  */

ssize_t
__libc_pread64 (int fd, void *buf, size_t nbytes, off64_t offset)
{
    static ssize_t max_read_nbytes = -1;
    ssize_t bytes_read;

    if (buf == NULL)
    {
	__set_errno (EINVAL);
	return -1;
    }

    bytes_read = 0;
    do
    {
	ssize_t toread, justread, toread_cp;
	int error;

	toread = nbytes - bytes_read;

	if (max_read_nbytes != -1 && toread > max_read_nbytes)
	{
#if 0
	    fprintf(stderr, "Adjusting pread64 of %d bytes from %d to %d\n",
		    nbytes, toread, max_read_nbytes);
#endif
	    toread = max_read_nbytes;
	}

	toread_cp = toread;
	justread = unix_pread64(fd, buf + bytes_read, &toread_cp, offset +
				bytes_read);

	if ((error = __zoid_error()))
	{
	    if (error != E2BIG)
	    {
		/* For a generic ZOID error, just bail out.  */
		__set_errno(error);
		return -1;
	    }
#if 0
	    fprintf(stderr, "Attempt to pread64 %d bytes returned excessive "
		    "size error by %d bytes\n", toread,
		    __zoid_excessive_size());
#endif
	    /* We tried to send a too large read request.  Adjust.  */
	    max_read_nbytes = toread - __zoid_excessive_size();
	}
	else
	{
	    if (justread < 0)
	    {
		/* For a generic pread64() error, just bail out.  */
		__set_errno(-justread);
		return -1;
	    }

	    bytes_read += justread;

	    if (justread != toread)
		/* pread64() as such succeeded, but it read fewer bytes than
		   expected.  We terminate prematurely then.  */
		break;
	}
    } while (bytes_read < nbytes);

    return bytes_read;
}

weak_alias (__libc_pread64, __pread64)
weak_alias (__libc_pread64, pread64)
