/****************************************************************************/
/* ZEPTOOS:zepto-info */
/*     This file is part of ZeptoOS: The Small Linux for Big Computers.
 *     See www.mcs.anl.gov/zeptoos for more information.
 */
/* ZEPTOOS:zepto-info */
/* */
/* ZEPTOOS:zepto-fillin */
/*     $Id: recvfrom.c,v 1.5 2007/06/27 15:20:38 iskra Exp $
 *     ZeptoOS_Version: 1.2
 *     ZeptoOS_Heredity: FOSS_MOD
 *     ZeptoOS_License: GPL
 */
/* ZEPTOOS:zepto-fillin */
/* */
/* ZEPTOOS:zepto-gpl */
/*      Copyright: Argonne National Laboratory, Department of Energy,
 *                 and UChicago Argonne, LLC.  2004, 2005, 2006, 2007
 *      ZeptoOS License: GPL
 * 
 *      This software is free.  See the file ZeptoOS/misc/license.GPL
 *      for complete details on your rights to copy, modify, and use this
 *      software.
 */
/* ZEPTOOS:zepto-gpl */
/****************************************************************************/

#include <errno.h>
#include <stddef.h>
#include <sys/socket.h>

#include "unix.h"

ssize_t
__libc_recvfrom (int fd, void *buf, size_t n, int flags, __SOCKADDR_ARG addr,
		 socklen_t *addr_len)
{
    static ssize_t max_recv_nbytes = -1;
    ssize_t bytes_recv;

    if (buf == NULL)
    {
	__set_errno (EINVAL);
	return -1;
    }

    bytes_recv = 0;
    do
    {
	ssize_t torecv, justrecv, torecv_cp;
	int error;

	torecv = n - bytes_recv;

	if (max_recv_nbytes != -1 && torecv > max_recv_nbytes)
	{
#if 0
	    fprintf(stderr, "Adjusting recvfrom of %d bytes from %d to %d\n",
		    n, torecv, max_recv_nbytes);
#endif
	    torecv = max_recv_nbytes;
	}

	torecv_cp = torecv;
	justrecv = unix_recvfrom(fd, buf + bytes_recv, &torecv_cp, flags,
				 addr.__sockaddr__, addr_len);
	if ((error = __zoid_error()))
	{
	    if (error != E2BIG)
	    {
		/* For a generic ZOID error, just bail out.  */
		__set_errno(error);
		return -1;
	    }
#if 0
	    fprintf(stderr, "Attempt to recvfrom %d bytes returned excessive "
		    "size error by %d bytes\n", torecv,
		    __zoid_excessive_size());
#endif
	    /* We tried to send a too large recvfrom request.  Adjust.  */
	    max_recv_nbytes = torecv - __zoid_excessive_size();
	}
	else
	{
	    if (justrecv < 0)
	    {
		/* For a generic recvfrom() error, just bail out.  */
		__set_errno(-justrecv);
		return -1;
	    }

	    bytes_recv += justrecv;

	    if (justrecv != torecv)
		/* recvfrom() as such succeeded, but it read fewer bytes than
		   expected.  We terminate prematurely then.  */
		break;
	}
    } while (bytes_recv < n);

    return bytes_recv;
}

weak_alias (__libc_recvfrom, recvfrom)
