/****************************************************************************/
/* ZEPTOOS:zepto-info */
/*     This file is part of ZeptoOS: The Small Linux for Big Computers.
 *     See www.mcs.anl.gov/zeptoos for more information.
 */
/* ZEPTOOS:zepto-info */
/* */
/* ZEPTOOS:zepto-fillin */
/*     $Id: write.c,v 1.6 2007/06/27 15:20:38 iskra Exp $
 *     ZeptoOS_Version: 1.2
 *     ZeptoOS_Heredity: FOSS_MOD
 *     ZeptoOS_License: GPL
 */
/* ZEPTOOS:zepto-fillin */
/* */
/* ZEPTOOS:zepto-gpl */
/*      Copyright: Argonne National Laboratory, Department of Energy,
 *                 and UChicago Argonne, LLC.  2004, 2005, 2006, 2007
 *      ZeptoOS License: GPL
 * 
 *      This software is free.  See the file ZeptoOS/misc/license.GPL
 *      for complete details on your rights to copy, modify, and use this
 *      software.
 */
/* ZEPTOOS:zepto-gpl */
/****************************************************************************/

#include <errno.h>
#include <stddef.h>
#include <unistd.h>

#include "unix.h"

//#include <stdio.h>

/* Note: most of the code below is duplicated in read.c, and p/p64 variants
   of write and read.  If anything needs to be changed in one of them, it
   probably also should be changed in all the others!  */

ssize_t
__libc_write (int fd, const void *buf, size_t nbytes)
{
    static ssize_t max_write_nbytes = -1;
    ssize_t bytes_written;

    if (buf == NULL)
    {
	__set_errno (EINVAL);
	return -1;
    }

    bytes_written = 0;
    do
    {
	ssize_t towrite, written;
	int error;

	towrite = nbytes - bytes_written;

	if (max_write_nbytes != -1 && towrite > max_write_nbytes)
	{
#if 0
	    fprintf(stderr, "Adjusting write of %d bytes from %d to %d\n",
		    nbytes, towrite, max_write_nbytes);
#endif
	    towrite = max_write_nbytes;
	}

	written = unix_write(fd, buf + bytes_written, towrite);

	if ((error = __zoid_error()))
	{
	    if (error != E2BIG)
	    {
		/* For a generic ZOID error, just bail out.  */
		__set_errno(error);
		return -1;
	    }
#if 0
	    fprintf(stderr, "Attempt to write %d bytes returned excessive "
		    "size error by %d bytes\n", towrite,
		    __zoid_excessive_size());
#endif
	    /* We tried to send a too large write request.  Adjust.  */
	    max_write_nbytes = towrite - __zoid_excessive_size();
	}
	else
	{
	    if (written < 0)
	    {
		/* For a generic write() error, just bail out.  */
		__set_errno(-written);
		return -1;
	    }

	    bytes_written += written;

	    if (written != towrite)
		/* write() as such succeeded, but it wrote fewer bytes than
		   expected.  We terminate prematurely then.  */
		break;
	}
    } while (bytes_written < nbytes);

    return bytes_written;
}

weak_alias (__libc_write, __write)
weak_alias (__libc_write, write)
